<?php

/**
 * It returns an object that provides access to the funcionality of Facebook API.
 *
 * Facebook provides appId and secret for kPAX.
 *
 * @return object Facebook API object 
 */
function facebook_api() {
	$config = array();
	
	// Facebook config for kPAX
	$config['appId'] = '439128496182481';
	$config['secret'] = '17359e04bcd696af72c20efde6575dd3';

	return new Facebook($config);
}

/**
 * It gets Facebook user data.
 *
 * If there is a Facebbok user logged in, it gets his profile data. Otherwise, it gets the Facebook login 
 * URL for the redirect.
 *
 * @return array Facebook user data
 */
function facebookKpax_get_fbk_data() {
	$fbk_data=array();
	$facebook = facebook_api();
	$fbk_user = $facebook->getUser();
	
	// If a Facebook user is logged in, it tries to get Facebbok user data
	if($fbk_user) {
		try {
			$fbk_data['user_profile']= $facebook->api('/me');
		} catch (FacebookApiException $e) {
			$fbk_user = null;
		}
	}

	// If no Facebook user logged in, it gets login URL
	if($fbk_user) {
		$fbk_data['login_url']='';
	} else {
		$fbk_data['login_url'] = $facebook->getLoginUrl(array('canvas' => 1, 'fbconnect' => 0, 'scope'=> 'email'));
	}
	return $fbk_data;
}

/**
 * Used to login a user with Facebook.
 *
 * If no Facebook user logged in, it redirects to the login URL. Otherwise, if the user exists in elgg, it 
 * logs him in elgg. If not exists, it creates a new elgg user with Facebook profile info, and then it logs
 * him in. 
 */
function facebookKpax_login() {

	if(elgg_is_logged_in()) {
		forward();
	}

	$facebook = facebook_api();
	$fbk_data = facebookKpax_get_fbk_data();

	if(isset($_GET['error'])) {
		forward();
	} else if(!empty($fbk_data['login_url'])) {
		// If no Facebook user logged in, redirects to the login URL		
		forward($fbk_data['login_url'], 'facebookKpax');
	} else if(empty($fbk_data['user_profile']['id'])) {
		forward();
	} else {
		// It checks if Facebook user already exists in elgg
		$options = array(
			'type' => 'user',
			'plugin_user_setting_name_value_pairs' => array(
				'fbid' => $fbk_data['user_profile']['id']
				),
			'plugin_user_setting_name_value_pairs_operator' => 'OR',
			'limit' => 0
		);
		$users = elgg_get_entities_from_plugin_user_settings($options);
		if ($users) {
			if (count($users) == 1) {
				try {
					// If Facebook user exists in elgg, then login it in elgg
					login($users[0]);
					system_message(elgg_echo('facebookKpax:login:success'));
					
					// It updates elgg email from Facebook
					$user = get_entity($users[0]->guid);
					$user->email = $fbk_data['user_profile']['email'];
					$user->save();

					// It updates avatar from Facebook profile image
					facebookKpax_update_avatar($user, $fbk_data['user_profile']['id']);

				} catch (LoginException $e) {
					register_error($e->getMessage());
					forward(REFERER);
				}
			} else {
				system_message(elgg_echo('facebookKpax:login:error'));
			}
			forward();
		} else {
			// If Facebook user don't exist in elgg, then create it
			$user = facebookKpax_create_user($fbk_data);

			// Login new user in elgg
			try {
				login($user);
				system_message(elgg_echo('facebookKpax:login:success'));
				forward();
			} catch (LoginException $e) {
				register_error($e->getMessage());
				forward(REFERER);
			}

		}

		register_error(elgg_echo('facebookKpax:login:error'));
		forward();
	}
}

/**
 * It creates an elgg user using Facebook profile data.
 *
 * The function uses name, username, email and profile image from Facebook.
 *
 * @param array $fbData Facebook profile data of user
 * @return object User created
 */
function facebookKpax_create_user($fbk_data) {
	$facebook = facebook_api();
	
	// It gets user data from Facebook profile	
	$username = $fbk_data['user_profile']['username'];
	$name = $fbk_data['user_profile']['name'];
	$email= $fbk_data['user_profile']['email'];

	// It generates a new random password for the user
	$password = generate_random_cleartext_password();

	// It creates the new elgg user
	$user = new ElggUser();
	$user->username = $username;
	$user->name = $name;
	$user->email = $email;
	$user->access_id = ACCESS_PUBLIC;
	$user->salt = generate_random_cleartext_password();
	$user->password = generate_user_password($user, $password);
	$user->owner_guid = 0;
	$user->container_guid = 0;
	if (!$user->save()) {
		register_error(elgg_echo('registerbad'));
		forward();
	} else {
		// It updates user avatar from Facebook profile image
		facebookKpax_update_avatar($user, $fbk_data['user_profile']['id']);
	}
	
	// It stores Facebook id of the user in the plugin user settings
	elgg_set_plugin_user_setting('fbid', $fbk_data['user_profile']['id'], $user->guid);

	return $user;
}
/**
 * It updates user's avatar from Facebook.
 *
 * @param object $user Elgg User object
 * @param string $facebookId Facebook Id for the user
 */
function facebookKpax_update_avatar($user, $fbk_id) {
	global $CONFIG;

	// It creates a temp file with user's Facebook profile image
	$img_url='https://graph.facebook.com/' . $fbk_id .'/picture?type=large';
	$temp=$CONFIG->dataroot.$user->getGUID().'.jpg';
	$img_content = file_get_contents($img_url);
	$fp = fopen($temp, "w");
	fwrite($fp, $img_content);
	fclose($fp);

	// It creates a new image for each icon size in elgg	
	$iconSizes = elgg_get_config('icon_sizes');
	$file = new ElggFile();
	$file->owner_guid = $user->getGUID();
	foreach ($iconSizes as $size => $dimensions)
	{
		$image = get_resized_image_from_existing_file(
			$temp,
			$dimensions[w],
			$dimensions[h],
			$dimensions[square]
		);
		$userId = $user->guid;
		$file->setFilename("profile/{$userId}{$size}.jpg");
		$file->open('write');
		$file->write($image);
		$file->close();
	}

	// It updates user's icontime
	$user->icontime = time();

	// It deletes temp file
	unlink($temp);
}

/**
 * It checks if a elgg user comes from Facebook.
 *
 * It checks if the user has a Facebook id stored.
 *
 * @param string $userName Elgg user name
 * @return bool True if Facebook user, false otherwise
 */
function facebookKpax_is_facebook_user($username) {
	$user = get_user_by_username($username);
	if ($user) {
		if (!elgg_get_plugin_user_setting('fbid',$user->guid,'facebookKpax'))
			return false;
		else
			return true;
	} else
		return false;
}

